/*
 * (C) Copyright 2014,2016 Hewlett Packard Enterprise Development LP
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package monasca.common.model.alarm;

public class AlarmNotificationMethodType {

  public static final String EMAIL = "EMAIL";
  public static final String WEBHOOK = "WEBHOOK";
  public static final String PAGERDUTY = "PAGERDUTY";
  
  public static String valueOf(String alarmType)
  {
	return alarmType.toUpperCase();
  }
}
